/*  PSX Controller Decoder Library (DenshaMascon.cpp)
	Written by: Kevin Ahrendt June 22nd, 2008
	Modified by: Yaasan / Desktop Station Dec 15, 2015
	
	Controller protocol implemented using Andrew J McCubbin's analysis.
	http://www.gamesx.com/controldata/psxcont/psxcont.htm
	
	Shift command is based on tutorial examples for ShiftIn and ShiftOut
	functions both written by Carlyn Maw and Tom Igoe
	http://www.arduino.cc/en/Tutorial/ShiftIn
	http://www.arduino.cc/en/Tutorial/ShiftOut

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*  
    
	E 1000000000001111, 1111000000001111
	8 1011000000001111, 1111000000001111
	7 1001000000001111, 1111000000001111
	6 0110000000001111, 1111000000001111
	5 0100000000001111, 1111000000001111
	4 0110000000001111, 1111000000001111
	3 1100000000001111, 1111000000001111
	2 1110000000001111, 1111000000001111
	1 0111000000001010, 1111000000001111
	N 0111000000001111, 1111000000001111
	1 1101100000001110, 1101000000001010
	2 1101100000001110, 1101000000001010
	3 1101000000001110, 1101000000001010
	4 1101100000001011, 1101000000001010, 
	5 1101000000001011, 1101000000001010

	       1101000000001010
	SELECT ........x.......
	START  ...........x....
	A      .......x
	B      ......x
	C      .....x
	MASK   zzzzz.......zzzz
    
    
*/

#include "DenshaMascon.h"

byte Mascon::shift(byte _dataOut)							// Does the actual shifting, both in and out simultaneously
{
	_temp = 0;
	_dataIn = 0;

	for (_i = 0; _i <= 7; _i++)
	{
		
		
		if ( _dataOut & (1 << _i) ) digitalWrite(_cmndPin, HIGH);	// Writes out the _dataOut bits
		else digitalWrite(_cmndPin, LOW);

		digitalWrite(_clockPin, LOW);
		
		delayMicroseconds(_delay);

		_temp = digitalRead(_dataPin);					// Reads the data pin
		if (_temp)
		{
			_dataIn = _dataIn | (B10000000 >> _i);		// Shifts the read data into _dataIn
		}

		digitalWrite(_clockPin, HIGH);
		delayMicroseconds(_delay);
	}
	return _dataIn;
}


void Mascon::setupPins(byte dataPin, byte cmndPin, byte attPin, byte clockPin, byte delay)
{
	pinMode(dataPin, INPUT);
	digitalWrite(dataPin, HIGH);	// Turn on internal pull-up
	_dataPin = dataPin;

	pinMode(cmndPin, OUTPUT);
	_cmndPin = cmndPin;

	pinMode(attPin, OUTPUT);
	_attPin = attPin;
	digitalWrite(_attPin, HIGH);

	pinMode(clockPin, OUTPUT);
	_clockPin = clockPin;
	digitalWrite(_clockPin, HIGH);
	
	_delay = delay;
}


unsigned int Mascon::read()
{
	digitalWrite(_attPin, LOW);

	shift(0x01);
	shift(0x42);
	shift(0xFF);

	_data1 = ~shift(0xFF);
	_data2 = ~shift(0xFF);

	digitalWrite(_attPin, HIGH);

	_dataOut = (_data2 << 8) | _data1;

	return _dataOut;
}


byte Mascon::Interval()
{
	int aSpeedRef;
	byte aRet = 0;
	
	
	aSpeedRef = gSpeedRef + (gPower * 10);
	
	//Limit
	if( aSpeedRef < 0)
	{
		aSpeedRef = 0;
	}
	else if ( aSpeedRef > 2048)
	{
		aSpeedRef = 2048;
	}
	
	if( (aSpeedRef >> 1) != (gSpeedRef >> 1))
	{
		aRet = 1;
	}
	
	gSpeedRef = aSpeedRef;
	
	return aRet;
	
}

word Mascon::GetSpeedValue()
{
	return gSpeedRef >> 1;
	
}

int Mascon::GetHandleValue()
{
	return gPower;
	
}

byte Mascon::GetButtonStatus(unsigned int inData)
{
	byte aRet = 0;
		
	if( (inData & BUTTON_E) > 0)
	{
		//Serial.print("SELECT,");
		aRet = aRet | 0b00001;
	}
	if( (inData & BUTTON_S) > 0)
	{
		//Serial.print("START,");  
		aRet = aRet | 0b00010;
	}
	if( (inData & BUTTON_A) > 0)
	{
		//Serial.print("A,");  
		aRet = aRet | 0b00100;
	}
	if( (inData & BUTTON_B) > 0)
	{
		//Serial.print("B,");  
		aRet = aRet | 0b01000;
	}
	if( (inData & BUTTON_C) > 0)
	{
		//Serial.print("C,");  
		aRet = aRet | 0b10000;
	}
	
	return aRet;

}


byte Mascon::GetHandleStatus(unsigned int inData)
{
	
	if( (inData & MASK_BTN)  == DEC_FLAG)
	{
		gDirection = 1;
	}
	else if( (inData & MASK_BTN)  == ACC_FLAG)
	{
		gDirection = 0;
	}
	
	
	if( (inData & MASK_BTN) == GEAR_EMG)
	{
		//Serial.print("EMG,");  
		gPower = -1024;
	}
	else if ( (inData & MASK_BTN) == GEAR_B8)
	{
		//Serial.print("B8,");  
		gPower = -8;
	}
	else if ( (inData & MASK_BTN) == GEAR_B7)
	{
		//Serial.print("B7,");  
		gPower = -7;
	}
	else if ( (inData & MASK_BTN) == GEAR_B6)
	{
		//Serial.print("B6,");  
		gPower = -6;
	}
	else if ( (inData & MASK_BTN) == GEAR_B5)
	{
		//Serial.print("B5,");  
		gPower = -5;
	}
	else if ( (inData & MASK_BTN) == GEAR_B4)
	{
		//Serial.print("B4,");  
		gPower = -4;
	}
	else if ( (inData & MASK_BTN) == GEAR_B3)
	{
		//Serial.print("B3,");  
		gPower = -3;
	}
	else if ( (inData & MASK_BTN) == GEAR_B2)
	{
		//Serial.print("B2,");  
		gPower = -2;
	}
	else if ( (inData & MASK_BTN) == GEAR_B1)
	{
		//Serial.print("B1,");  
		gPower = -1;
	}
	else if ( (inData & MASK_BTN) == GEAR_N)
	{
		//Serial.print("N,");  
		gPower = 0;
	}
	else if ( (inData & MASK_BTN) == GEAR_P1)
	{
		//Serial.print("P1,");  
		gPower = 1;
	}
	else if ( (inData & MASK_BTN) == GEAR_P2)
	{
		//Serial.print("P2,");  
		gPower = 2;
	}
	else if ( (inData & MASK_BTN) == GEAR_P3)
	{
		//Serial.print("P3,");  
		gPower = 3;
	}
	else if ( (inData & MASK_BTN) == GEAR_P4)
	{
		//Serial.print("P4,");  
		gPower = 4;
	}
	else if ( (inData & MASK_BTN) == GEAR_P5)
	{
		//Serial.print("P5,");  
		gPower = 5;
	}		
}


void Mascon::PutButtonBuffer(unsigned int inData)
{
	gPsxData[2] = gPsxData[1];
	gPsxData[1] = gPsxData[0];
	gPsxData[0] = inData;
}

byte Mascon::CheckButtonBuffer(unsigned int inData)
{
	byte aOk = 0;
	
	for( int i = 0; i < 3; i++)
	{
		if( gPsxData[i] == inData)
		{
			aOk++;
		}
	}
	
	return (aOk >= 2) ? 1 : 0;
}